"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path = _interopRequireDefault(require("path"));

var _torchlight = _interopRequireDefault(require("../torchlight"));

var _block = _interopRequireDefault(require("../block"));

var _cheerio = _interopRequireDefault(require("cheerio"));

var _chokidar = _interopRequireDefault(require("chokidar"));

var _log = _interopRequireDefault(require("../support/log"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _bus = require("../support/bus");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _default(torchlight, options) {
  options = {
    input: torchlight.config('highlight.input', ''),
    output: torchlight.config('highlight.output', ''),
    include: torchlight.config('highlight.includeGlobs', ['**/*.htm', '**/*.html']),
    exclude: torchlight.config('highlight.excludePatterns', ['/node_modules/', '/vendor/']),
    watch: false,
    ...options
  };

  if (options.watch) {
    _log.default.info(`
***************************************
*   Torchlight is watching files...   *
***************************************
`);
  }

  const input = _path.default.resolve(options.input);

  const output = _path.default.resolve(options.output || options.input);

  const watcher = _chokidar.default.watch(normalizeStringedArray(options.include), {
    cwd: input,
    ignored: path => normalizeStringedArray(options.exclude).some(s => path.includes(s)),
    ignoreInitial: false
  });

  watcher.on('all', (event, file) => {
    if (event !== 'add' && event !== 'change') {
      return;
    }

    _log.default.info('Highlighting %s', file);

    const source = _fsExtra.default.readFileSync(_path.default.join(input, file), 'utf-8');

    highlight(torchlight, source).then(highlighted => {
      const destination = _path.default.join(output, file);

      _fsExtra.default.ensureFileSync(destination);

      if (highlighted === _fsExtra.default.readFileSync(destination, 'utf-8')) {
        return;
      }

      _log.default.info('Writing to %s', destination);

      _fsExtra.default.writeFileSync(destination, highlighted, 'utf-8');
    });
  });
  watcher.on('ready', function () {
    if (!options.watch) {
      watcher.close().then(() => _bus.bus.emit(_bus.FILE_WATCHING_COMPLETE));
    }
  });
}

function normalizeStringedArray(value) {
  return (typeof value === 'string' ? value.split(',') : value).filter(x => x);
}

function highlight(torchlight, source) {
  let highlighted = source;

  const $ = _cheerio.default.load(source, {
    sourceCodeLocationInfo: true
  }, false);

  const blocks = []; // Search for blocks that have not already been processed.

  $('pre:not([data-torchlight-processed])').each((index, pre) => {
    const $pre = $(pre);
    $pre.children('code').each((index, code) => {
      const $code = $(code);
      const block = new _block.default({
        // Using `text()` will re-encode entities like &lgt;
        code: $code.text(),
        language: decipherLanguage($pre, $code)
      }); // Add our class placeholder as a class, so that we don't overwrite
      // any classes that are already there.

      $pre.addClass(block.placeholder('class')); // Add a fake style that we can replace later.

      $pre.css(block.placeholder('style'), '0'); // Store the raw code as the developer wrote it, so we can re-highlight
      // it later if we need to, or allow it to be copied to clipboard.

      const raw = `<textarea data-torchlight-original='true' style='display: none !important;'>${$code.html()}</textarea>`; // Add the placeholder inside the code tag.

      $code.html(block.placeholder('highlighted') + raw); // Give the developer an opportunity to add things to the placeholder
      // element. Like copy to clipboard buttons, language indicators, etc.

      if (torchlight.config('modifyPlaceholderElement')) {
        torchlight.config('modifyPlaceholderElement')($, $pre, $code, block);
      }

      blocks.push(block);
    }); // Add the options hash that this block will be highlighted with.

    $pre.attr('data-torchlight-processed', torchlight.configHash()); // Cut out the *exact* pre element as it is in the file. Cheerio converts
    // single quotes to double, normalizes whitespace, and otherwise "cleans
    // up" the parsed document, so we can't simply modify the Cheerio dom and
    // write it back to disk. Instead we're going to surgically cut out the
    // pre tag and all its contents without touching anything else around it.

    const pristinePreElement = source.substring(pre.sourceCodeLocation.startOffset, pre.sourceCodeLocation.endOffset); // Swap out the original tag with the outerHTML of our modified tag.

    highlighted = highlighted.replace(pristinePreElement, $.html($pre));
  });

  if (!blocks.length) {
    return Promise.resolve(source);
  }

  return torchlight.highlight(blocks).then(() => {
    blocks.forEach(block => {
      var _block$classes, _block$styles;

      const swap = {
        [block.placeholder('class')]: (_block$classes = block.classes) !== null && _block$classes !== void 0 ? _block$classes : '',
        [block.placeholder('style') + ': 0;']: (_block$styles = block.styles) !== null && _block$styles !== void 0 ? _block$styles : '',
        [block.placeholder('highlighted')]: block.highlighted
      };
      Object.keys(swap).forEach(key => {
        highlighted = highlighted.replace(key, () => swap[key]);
      });
    });
    return highlighted;
  });
}
/**
 * Given a <pre> element, figure out what language it is.
 *
 * @param $pre
 * @return {string}
 */


function decipherLanguage($pre, $code) {
  const custom = _torchlight.default.config('highlight.decipherLanguageFromElement'); // Let the developer add their own deciphering mechanism.


  if (custom) {
    const lang = custom($pre);

    if (lang) {
      return lang;
    }
  }

  const langs = [// Look first at the code element.
  ...decipherFromElement($code), // And then the pre element.
  ...decipherFromElement($pre)];
  return langs.length ? langs[0] : 'text';
}
/**
 * Given any element, figure out what language it might be.
 *
 * @param $el
 * @return {*[]}
 */


function decipherFromElement($el) {
  var _$el$data, _$el$data2;

  if (!$el) {
    return [];
  }

  const classes = ($el.attr('class') || '').split(' ') // These classes are commonly used to denote code languages.
  .filter(c => c.startsWith('language-') || c.startsWith('lang-')).map(c => c.replace('language-', '').replace('lang-', ''));
  return [// Data attributes get highest priority.
  (_$el$data = $el.data()) === null || _$el$data === void 0 ? void 0 : _$el$data.language, (_$el$data2 = $el.data()) === null || _$el$data2 === void 0 ? void 0 : _$el$data2.lang, ...classes].filter(l => l);
}